@@ -1,11 +1,9 @@
 package Archive::Zip;
 
+use 5.006;
 use strict;
-BEGIN {
-    require 5.003_96;
-}
-use UNIVERSAL           ();
 use Carp                ();
+use Cwd                 ();
 use IO::File            ();
 use IO::Seekable        ();
 use Compress::Raw::Zlib ();
@@ -15,7 +13,7 @@ use FileHandle          ();
 
 use vars qw( $VERSION @ISA );
 BEGIN {
-    $VERSION = '1.29';
+    $VERSION = '1.32';
 
     require Exporter;
     @ISA = qw( Exporter );
@@ -374,7 +372,7 @@ sub _binmode {
 
 # Attempt to guess whether file handle is seekable.
 # Because of problems with Windows, this only returns true when
-# the file handle is a real file.  
+# the file handle is a real file.
 sub _isSeekable {
     my $fh = shift;
     return 0 unless ref $fh;
@@ -402,7 +400,7 @@ sub _isSeekable {
         ) ? 1 : 0;
 }
 
-# Print to the filehandle, while making sure the pesky Perl special global 
+# Print to the filehandle, while making sure the pesky Perl special global
 # variables don't interfere.
 sub _print
 {
@@ -480,7 +478,7 @@ sub _readSignature {
 }
 
 # Utility method to make and open a temp file.
-# Will create $temp_dir if it doesn't exist.
+# Will create $temp_dir if it does not exist.
 # Returns file handle and name:
 #
 # my ($fh, $name) = Archive::Zip::tempFile();
@@ -491,7 +489,7 @@ sub tempFile {
     my $dir = ( ref( $_[0] ) eq 'HASH' ) ? shift->{tempDir} : shift;
     my ( $fh, $filename ) = File::Temp::tempfile(
         SUFFIX => '.zip',
-        UNLINK => 0,        # we will delete it!
+        UNLINK => 1,
         $dir ? ( DIR => $dir ) : ()
     );
     return ( undef, undef ) unless $fh;
@@ -500,7 +498,7 @@ sub tempFile {
 }
 
 # Return the normalized directory name as used in a zip file (path
-# separators become slashes, etc.). 
+# separators become slashes, etc.).
 # Will translate internal slashes in path components (i.e. on Macs) to
 # underscores.  Discards volume names.
 # When $forceDir is set, returns paths with trailing slashes (or arrays
@@ -514,10 +512,10 @@ sub tempFile {
 # ./a/b         ('a','b')   a/b
 # ./a/b/        ('a','b')   a/b
 # a/b/          ('a','b')   a/b
-# /a/b/         ('','a','b')    /a/b
-# c:\a\b\c.doc  ('','a','b','c.doc')    /a/b/c.doc      # on Windoze
+# /a/b/         ('','a','b')    a/b
+# c:\a\b\c.doc  ('','a','b','c.doc')    a/b/c.doc      # on Windows
 # "i/o maps:whatever"   ('i_o maps', 'whatever')  "i_o maps/whatever"   # on Macs
-sub _asZipDirName    
+sub _asZipDirName
 {
     my $name      = shift;
     my $forceDir  = shift;
@@ -529,14 +527,25 @@ sub _asZipDirName
     if ( @dirs > 0 ) { pop (@dirs) unless $dirs[-1] }   # remove empty component
     push ( @dirs, defined($file) ? $file : '' );
     #return wantarray ? @dirs : join ( '/', @dirs );
-    return join ( '/', @dirs );
+
+    my $normalised_path = join '/', @dirs;
+
+    # Leading directory separators should not be stored in zip archives.
+    # Example:
+    #   C:\a\b\c\      a/b/c
+    #   C:\a\b\c.txt   a/b/c.txt
+    #   /a/b/c/        a/b/c
+    #   /a/b/c.txt     a/b/c.txt
+    $normalised_path =~ s{^/}{};  # remove leading separator
+
+    return $normalised_path;
 }
 
 # Return an absolute local name for a zip name.
 # Assume a directory if zip name has trailing slash.
 # Takes an optional volume name in FS format (like 'a:').
 #
-sub _asLocalName    
+sub _asLocalName
 {
     my $name   = shift;    # zip format
     my $volume = shift;
@@ -547,9 +556,8 @@ sub _asLocalName
     $filename = '' unless defined($filename);
     my $localDirs = @paths ? File::Spec->catdir(@paths) : '';
     my $localName = File::Spec->catpath( $volume, $localDirs, $filename );
-    require Cwd;
     unless ( $volume ) {
-        $localName = File::Spec->rel2abs( $localName, Cwd::getcwd );
+        $localName = File::Spec->rel2abs( $localName, Cwd::getcwd() );
     }
     return $localName;
 }
@@ -560,6 +568,8 @@ __END__
 
 =pod
 
+=encoding utf8
+
 =head1 NAME
 
 Archive::Zip - Provide an interface to ZIP archive files.
@@ -569,28 +579,28 @@ Archive::Zip - Provide an interface to ZIP archive files.
    # Create a Zip file
    use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
    my $zip = Archive::Zip->new();
-   
+
    # Add a directory
    my $dir_member = $zip->addDirectory( 'dirname/' );
-   
+
    # Add a file from a string with compression
    my $string_member = $zip->addString( 'This is a test', 'stringMember.txt' );
    $string_member->desiredCompressionMethod( COMPRESSION_DEFLATED );
-   
+
    # Add a file from disk
    my $file_member = $zip->addFile( 'xyz.pl', 'AnotherName.pl' );
-   
+
    # Save the Zip file
    unless ( $zip->writeToFileNamed('someZip.zip') == AZ_OK ) {
        die 'write error';
    }
-   
+
    # Read a Zip file
    my $somezip = Archive::Zip->new();
    unless ( $somezip->read( 'someZip.zip' ) == AZ_OK ) {
        die 'read error';
    }
-   
+
    # Change the compression type for a file in the Zip
    my $member = $somezip->memberNamed( 'stringMember.txt' );
    $member->desiredCompressionMethod( COMPRESSION_STORED );
@@ -650,7 +660,7 @@ if you call them with a single name.
 
 =head2 Archive::Zip Object Model
 
-=head2 Overview
+=head3 Overview
 
 Archive::Zip::Archive objects are what you ordinarily deal with.
 These maintain the structure of a zip file, without necessarily
@@ -722,9 +732,9 @@ as inline subroutines, using the C<use constant> pragma. They can be imported
 into your namespace using the C<:ERROR_CODES> tag:
 
   use Archive::Zip qw( :ERROR_CODES );
-  
+
   ...
-  
+
   unless ( $zip->read( 'myfile.zip' ) == AZ_OK ) {
       die "whoops!";
   }
@@ -771,10 +781,10 @@ compression method:
 
   my $member = $zip->memberNamed( 'xyz.txt' );
   $member->compressionMethod();    # return current compression
-  
+
   # set to read uncompressed
   $member->desiredCompressionMethod( COMPRESSION_STORED );
-  
+
   # set to read compressed
   $member->desiredCompressionMethod( COMPRESSION_DEFLATED );
 
@@ -805,26 +815,26 @@ The levels given can be:
 
 =over 4
 
-=item 0 or COMPRESSION_LEVEL_NONE
+=item * 0 or COMPRESSION_LEVEL_NONE
 
 This is the same as saying
 
   $member->desiredCompressionMethod( COMPRESSION_STORED );
 
-=item 1 .. 9
+=item * 1 .. 9
 
 1 gives the best speed and worst compression, and 9 gives the
 best compression and worst speed.
 
-=item COMPRESSION_LEVEL_FASTEST
+=item * COMPRESSION_LEVEL_FASTEST
 
 This is a synonym for level 1.
 
-=item COMPRESSION_LEVEL_BEST_COMPRESSION
+=item * COMPRESSION_LEVEL_BEST_COMPRESSION
 
 This is a synonym for level 9.
 
-=item COMPRESSION_LEVEL_DEFAULT
+=item * COMPRESSION_LEVEL_DEFAULT
 
 This gives a good compromise between speed and compression,
 and is currently equivalent to 6 (this is in the zlib code).
@@ -845,6 +855,8 @@ about this unless you're subclassing.
 
 =item new( [$fileName] )
 
+=item new( { filename => $fileName } )
+
 Make a new, empty zip archive.
 
     my $zip = Archive::Zip->new();
@@ -874,6 +886,8 @@ methods. Do not call them as class methods:
 
 =item Archive::Zip::computeCRC32( $string [, $crc] )
 
+=item Archive::Zip::computeCRC32( { string => $string [, checksum => $crc ] } )
+
 This is a utility function that uses the Compress::Raw::Zlib CRC
 routine to compute a CRC-32. You can get the CRC of a string:
 
@@ -887,6 +901,8 @@ Or you can compute the running CRC:
 
 =item Archive::Zip::setChunkSize( $number )
 
+=item Archive::Zip::setChunkSize( { chunkSize => $number } )
+
 Report or change chunk size used for reading and writing.
 This can make big differences in dealing with large files.
 Currently, this defaults to 32K. This also changes the chunk
@@ -907,6 +923,8 @@ Returns the current chunk size:
 
 =item Archive::Zip::setErrorHandler( \&subroutine )
 
+=item Archive::Zip::setErrorHandler( { subroutine => \&subroutine } )
+
 Change the subroutine called with error strings. This
 defaults to \&Carp::carp, but you may want to change it to
 get the error strings. This is not exportable, so you must
@@ -923,7 +941,9 @@ not just set it to a number):
 
     $Carp::CarpLevel++;
 
-=item Archive::Zip::tempFile( [$tmpdir] )
+=item Archive::Zip::tempFile( [ $tmpdir ] )
+
+=item Archive::Zip::tempFile( { tempDir => $tmpdir } )
 
 Create a uniquely named temp file. It will be returned open
 for read/write. If C<$tmpdir> is given, it is used as the
@@ -937,7 +957,7 @@ environment variable. But see the L<File::Spec|File::Spec>
 documentation for your system. Note that on many systems, if you're
 running in taint mode, then you must make sure that C<$ENV{TMPDIR}> is
 untainted for it to be used.
-Will I<NOT> create C<$tmpdir> if it doesn't exist (this is a change
+Will I<NOT> create C<$tmpdir> if it does not exist (this is a change
 from prior versions!). Returns file handle and name:
 
     my ($fh, $name) = Archive::Zip::tempFile();
@@ -966,11 +986,15 @@ Return a list of the (internal) file names of the zip members
 
 =item memberNamed( $string )
 
+=item memberNamed( { zipName => $string } )
+
 Return ref to member whose filename equals given filename or
 undef. C<$string> must be in Zip (Unix) filename format.
 
 =item membersMatching( $regex )
 
+=item membersMatching( { regex => $regex } )
+
 Return array of members whose filenames match given regular
 expression in list context. Returns number of matching
 members in scalar context.
@@ -1016,7 +1040,9 @@ Returns the offset into the zip file where the CD begins. Not
 used for writing zips, but might be interesting if you read a
 zip in.
 
-=item zipfileComment( [$string] )
+=item zipfileComment( [ $string ] )
+
+=item zipfileComment( [ { comment => $string } ] )
 
 Get or set the zipfile comment. Returns the old comment.
 
@@ -1060,15 +1086,20 @@ that names be unique within a zip (this is not enforced).
 
 =item removeMember( $memberOrName )
 
+=item removeMember( { memberOrZipName => $memberOrName } )
+
 Remove and return the given member, or match its name and
-remove it. Returns undef if member or name doesn't exist in this
+remove it. Returns undef if member or name does not exist in this
 Zip. No-op if member does not belong to this zip.
 
 =item replaceMember( $memberOrName, $newMember )
 
+=item replaceMember( { memberOrZipName => $memberOrName,
+    newMember => $newMember } )
+
 Remove and return the given member, or match its name and
 remove it. Replace with new member. Returns undef if member or
-name doesn't exist in this Zip, or if C<$newMember> is undefined.
+name does not exist in this Zip, or if C<$newMember> is undefined.
 
 It is an (undiagnosed) error to provide a C<$newMember> that is a
 member of the zip being modified.
@@ -1080,8 +1111,11 @@ member of the zip being modified.
 
 =item extractMember( $memberOrName [, $extractedName ] )
 
+=item extractMember( { memberOrZipName => $memberOrName
+    [, name => $extractedName ] } )
+
 Extract the given member, or match its name and extract it.
-Returns undef if member doesn't exist in this Zip. If
+Returns undef if member does not exist in this Zip. If
 optional second arg is given, use it as the name of the
 extracted member. Otherwise, the internal filename of the
 member is used as the name of the extracted file or
@@ -1093,9 +1127,12 @@ on success.
 
 =item extractMemberWithoutPaths( $memberOrName [, $extractedName ] )
 
+=item extractMemberWithoutPaths( { memberOrZipName => $memberOrName
+    [, name => $extractedName ] } )
+
 Extract the given member, or match its name and extract it.
 Does not use path information (extracts into the current
-directory). Returns undef if member doesn't exist in this
+directory). Returns undef if member does not exist in this
 Zip.
 If optional second arg is given, use it as the name of the
 extracted member (its paths will be deleted too). Otherwise,
@@ -1105,6 +1142,8 @@ on success.
 
 =item addMember( $member )
 
+=item addMember( { member => $member } )
+
 Append a member (possibly from another zip file) to the zip
 file. Returns the new member. Generally, you will use
 addFile(), addDirectory(), addFileOrDirectory(), addString(),
@@ -1116,13 +1155,18 @@ or read() to add members.
 
 =item updateMember( $memberOrName, $fileName )
 
+=item updateMember( { memberOrZipName => $memberOrName, name => $fileName } )
+
 Update a single member from the file or directory named C<$fileName>.
 Returns the (possibly added or updated) member, if any; C<undef> on
 errors.
 The comparison is based on C<lastModTime()> and (in the case of a
 non-directory) the size of the file.
 
-=item addFile( $fileName [, $newName ] )
+=item addFile( $fileName [, $newName, $compressionLevel ] )
+
+=item addFile( { filename => $fileName
+    [, zipName => $newName, compressionLevel => $compressionLevel } ] )
 
 Append a member whose data comes from an external file,
 returning the member or undef. The member will have its file
@@ -1140,13 +1184,15 @@ The text mode bit will be set if the contents appears to be
 text (as returned by the C<-T> perl operator).
 
 
-I<NOTE> that you shouldn't (generally) use absolute path names
+I<NOTE> that you should not (generally) use absolute path names
 in zip member names, as this will cause problems with some zip
 tools as well as introduce a security hole and make the zip
 harder to use.
 
 =item addDirectory( $directoryName [, $fileName ] )
 
+=item addDirectory( { directoryName => $directoryName
+    [, zipName => $fileName ] } )
 
 
 Append a member created from the given directory name. The
@@ -1160,8 +1206,10 @@ member (which defaults to C<$directoryName>). If given, it
 must be in Zip (Unix) format.
 Returns the new member.
 
-=item addFileOrDirectory( $name [, $newName ] )
+=item addFileOrDirectory( $name [, $newName, $compressionLevel ] )
 
+=item addFileOrDirectory( { name => $name [, zipName => $newName,
+    compressionLevel => $compressionLevel ] } )
 
 
 Append a member from the file or directory named $name. If
@@ -1172,9 +1220,10 @@ The optional second argument sets the name of the archive
 member (which defaults to C<$name>). If given, it must be in
 Zip (Unix) format.
 
-=item addString( $stringOrStringRef, $name )
-
+=item addString( $stringOrStringRef, $name, [$compressionLevel] )
 
+=item addString( { string => $stringOrStringRef [, zipName => $name,
+    compressionLevel => $compressionLevel ] } )
 
 Append a member created from the given string or string
 reference. The name is given by the second argument.
@@ -1186,6 +1235,8 @@ defaults.
 
 =item contents( $memberOrMemberName [, $newContents ] )
 
+=item contents( { memberOrZipName => $memberOrMemberName
+    [, contents => $newContents ] } )
 
 
 Returns the uncompressed data for a particular member, or
@@ -1214,7 +1265,7 @@ one.
 
 =item writeToFileNamed( $fileName )
 
-
+=item writeToFileNamed( { fileName => $fileName } )
 
 Write a zip archive to named file. Returns C<AZ_OK> on
 success.
@@ -1265,6 +1316,9 @@ archive.
 
 =item writeCentralDirectory( $fileHandle [, $offset ] )
 
+=item writeCentralDirectory( { fileHandle => $fileHandle
+    [, offset => $offset ] } )
+
 Writes the central directory structure to the given file
 handle.
 
@@ -1290,6 +1344,8 @@ directory:
 
 =item overwriteAs( $newName )
 
+=item overwriteAs( { filename => $newName } )
+
 Write the zip to the specified file, as safely as possible.
 This is done by first writing to a temp file, then renaming
 the original if it exists, then renaming the temp file, then
@@ -1304,6 +1360,8 @@ error.
 
 =item read( $fileName )
 
+=item read( { filename => $fileName } )
+
 Read zipfile headers from a zip file, appending new members.
 Returns C<AZ_OK> or error code.
 
@@ -1312,6 +1370,8 @@ Returns C<AZ_OK> or error code.
 
 =item readFromFileHandle( $fileHandle, $filename )
 
+=item readFromFileHandle( { fileHandle => $fileHandle, filename => $filename } )
+
 Read zipfile headers from an already-opened file handle,
 appending new members. Does not close the file handle.
 Returns C<AZ_OK> or error code. Note that this requires a
@@ -1335,31 +1395,34 @@ A usage example:
 
   use Archive::Zip;
   my $zip = Archive::Zip->new();
-  
+
   # add all readable files and directories below . as xyz/*
   $zip->addTree( '.', 'xyz' );
-  
+
   # add all readable plain files below /abc as def/*
   $zip->addTree( '/abc', 'def', sub { -f && -r } );
-  
+
   # add all .c files below /tmp as stuff/*
   $zip->addTreeMatching( '/tmp', 'stuff', '\.c$' );
-  
+
   # add all .o files below /tmp as stuff/* if they aren't writable
   $zip->addTreeMatching( '/tmp', 'stuff', '\.o$', sub { ! -w } );
-  
+
   # add all .so files below /tmp that are smaller than 200 bytes as stuff/*
   $zip->addTreeMatching( '/tmp', 'stuff', '\.o$', sub { -s < 200 } );
-  
+
   # and write them into a file
   $zip->writeToFileNamed('xxx.zip');
-  
+
   # now extract the same files into /tmpx
   $zip->extractTree( 'stuff', '/tmpx' );
 
 =over 4
 
-=item $zip->addTree( $root, $dest [,$pred] ) -- Add tree of files to a zip
+=item $zip->addTree( $root, $dest [, $pred, $compressionLevel ] ) -- Add tree of files to a zip
+
+=item $zip->addTree( { root => $root, zipName => $dest [, select => $pred,
+    compressionLevel => $compressionLevel ] )
 
 C<$root> is the root of the tree of files and directories to be
 added. It is a valid directory name on your system. C<$dest> is
@@ -1400,7 +1463,10 @@ check for the validity of filenames.
 Note that you generally I<don't> want to make zip archive member names
 absolute.
 
-=item $zip->addTreeMatching( $root, $dest, $pattern [,$pred] )
+=item $zip->addTreeMatching( $root, $dest, $pattern [, $pred, $compressionLevel ] )
+
+=item $zip->addTreeMatching( { root => $root, zipName => $dest, pattern =>
+    $pattern [, select => $pred, compressionLevel => $compressionLevel ] } )
 
 $root is the root of the tree of files and directories to be
 added $dest is the name for the root in the zip file (undef
@@ -1410,13 +1476,13 @@ optional subroutine reference to select files: it is passed
 the name of the prospective file or directory in C<$_>, and
 if it returns true, the file or directory will be included.
 The default is to add all readable files and directories. To
-add all files in and below the current dirctory whose names
+add all files in and below the current directory whose names
 end in C<.pl>, and make them extract into a subdirectory
 named C<xyz>, do this:
 
   $zip->addTreeMatching( '.', 'xyz', '\.pl$' )
 
-To add all I<writable> files in and below the dirctory named
+To add all I<writable> files in and below the directory named
 C</abc> whose names end in C<.pl>, and make them extract into
 a subdirectory named C<xyz>, do this:
 
@@ -1425,9 +1491,10 @@ a subdirectory named C<xyz>, do this:
 Returns AZ_OK on success. Note that this will not follow
 symbolic links to directories.
 
-=item $zip->updateTree( $root, [ $dest, [ $pred [, $mirror]]] );
-
+=item $zip->updateTree( $root [, $dest , $pred , $mirror, $compressionLevel ] );
 
+=item $zip->updateTree( { root => $root [, zipName => $dest, select => $pred,
+    mirror => $mirror, compressionLevel => $compressionLevel ] } );
 
 Update a zip file from a directory tree.
 
@@ -1436,31 +1503,17 @@ checks to see whether the file or directory already exists in the zip
 file, and whether it has been changed.
 
 If the fourth argument C<$mirror> is true, then delete all my members
-if corresponding files weren't found.
-
+if corresponding files were not found.
 
 Returns an error code or AZ_OK if all is well.
 
-=item $zip->extractTree()
-
-
-
-=item $zip->extractTree( $root )
-
-
-
-=item $zip->extractTree( $root, $dest )
-
-
-
-=item $zip->extractTree( $root, $dest, $volume )
-
+=item $zip->extractTree( [ $root, $dest, $volume } ] )
 
+=item $zip->extractTree( [ { root => $root, zipName => $dest, volume => $volume } ] )
 
 If you don't give any arguments at all, will extract all the
 files in the zip with their original names.
 
-
 If you supply one argument for C<$root>, C<extractTree> will extract
 all the members whose names start with C<$root> into the current
 directory, stripping off C<$root> first.
@@ -1472,13 +1525,10 @@ For instance,
 when applied to a zip containing the files:
 a/x a/b/c ax/d/e d/e will extract:
 
-
 a/x as ./x
 
-
 a/b/c as ./b/c
 
-
 If you give two arguments, C<extractTree> extracts all the members
 whose names start with C<$root>. It will translate C<$root> into
 C<$dest> to construct the destination file name.
@@ -1490,26 +1540,20 @@ For instance,
 when applied to a zip containing the files:
 a/x a/b/c ax/d/e d/e will extract:
 
-
 a/x to d/e/x
 
-
 a/b/c to d/e/b/c and ignore ax/d/e and d/e
 
-
 If you give three arguments, C<extractTree> extracts all the members
 whose names start with C<$root>. It will translate C<$root> into
 C<$dest> to construct the destination file name, and then it will
 convert to local file system format, using C<$volume> as the name of
 the destination volume.
 
-
 C<$root> and C<$dest> are in Zip (Unix) format.
 
-
 C<$volume> is in local file system format.
 
-
 For instance, under Windows,
 
    $zip->extractTree( 'a', 'd/e', 'f:' );
@@ -1517,13 +1561,10 @@ For instance, under Windows,
 when applied to a zip containing the files:
 a/x a/b/c ax/d/e d/e will extract:
 
-
 a/x to f:d/e/x
 
-
 a/b/c to f:d/e/b/c and ignore ax/d/e and d/e
 
-
 If you want absolute paths (the prior example used paths relative to
 the current directory on the destination volume, you can specify these
 in C<$dest>:
@@ -1533,22 +1574,36 @@ in C<$dest>:
 when applied to a zip containing the files:
 a/x a/b/c ax/d/e d/e will extract:
 
-
 a/x to f:\d\e\x
 
-
 a/b/c to f:\d\e\b\c and ignore ax/d/e and d/e
 
 Returns an error code or AZ_OK if everything worked OK.
 
 =back
 
+=head1 Archive::Zip Global Variables
+
+=over 4
+
+=item $Archive::Zip::UNICODE
+
+This variable governs how Unicode file and directory names are added
+to or extracted from an archive. If set, file and directory names are considered
+to be UTF-8 encoded. This is I<EXPERIMENTAL AND BUGGY (there are some edge cases
+on Win32)>. Please report problems.
+
+    {
+        local $Archive::Zip::UNICODE = 1;
+        $zip->addFile('Déjà vu.txt');
+    }
+
+=back
+
 =head1 MEMBER OPERATIONS
 
-
 =head2 Member Class Methods
 
-
 Several constructors allow you to construct members without adding
 them to a zip archive. These work the same as the addFile(),
 addDirectory(), and addString() zip instance methods described above,
@@ -1556,19 +1611,19 @@ but they don't add the new members to a zip.
 
 =over 4
 
-=item Archive::Zip::Member->newFromString( $stringOrStringRef [, $fileName] )
-
+=item Archive::Zip::Member->newFromString( $stringOrStringRef [, $fileName ] )
 
+=item Archive::Zip::Member->newFromString( { string => $stringOrStringRef
+    [, zipName => $fileName ] )
 
 Construct a new member from the given string. Returns undef
 on error.
 
     my $member = Archive::Zip::Member->newFromString( 'This is a test',
-                                                 'xyz.txt' );
-
-=item newFromFile( $fileName )
 
+=item newFromFile( $fileName [, $zipName ] )
 
+=item newFromFile( { filename => $fileName [, zipName => $zipName ] } )
 
 Construct a new member from the given file. Returns undef on
 error.
@@ -1577,18 +1632,17 @@ error.
 
 =item newDirectoryNamed( $directoryName [, $zipname ] )
 
-
+=item newDirectoryNamed( { directoryName => $directoryName
+    [, zipName => $zipname ] } )
 
 Construct a new member from the given directory.
-C<$directoryName> must be a valid name on your file system; it doesn't
+C<$directoryName> must be a valid name on your file system; it does not
 have to exist.
 
-
 If given, C<$zipname> will be the name of the zip member; it must be a
 valid Zip (Unix) name. If not given, it will be converted from
 C<$directoryName>.
 
-
 Returns undef on error.
 
     my $member = Archive::Zip::Member->newDirectoryNamed( 'CVS/' );
@@ -1597,41 +1651,32 @@ Returns undef on error.
 
 =head2 Member Simple accessors
 
-
 These methods get (and/or set) member attribute values.
 
 =over 4
 
 =item versionMadeBy()
 
-
-
 Gets the field from the member header.
 
-=item fileAttributeFormat( [$format] )
-
+=item fileAttributeFormat( [ $format ] )
 
+=item fileAttributeFormat( [ { format => $format ] } )
 
 Gets or sets the field from the member header. These are
 C<FA_*> values.
 
 =item versionNeededToExtract()
 
-
-
 Gets the field from the member header.
 
 =item bitFlag()
 
-
-
 Gets the general purpose bit field from the member header.
 This is where the C<GPBF_*> bits live.
 
 =item compressionMethod()
 
-
-
 Returns the member compression method. This is the method
 that is currently being used to compress the member data.
 This will be COMPRESSION_STORED for added string or file
@@ -1640,9 +1685,9 @@ from a zip file. However, this module can only handle members
 whose data is in COMPRESSION_STORED or COMPRESSION_DEFLATED
 format.
 
-=item desiredCompressionMethod( [$method] )
-
+=item desiredCompressionMethod( [ $method ] )
 
+=item desiredCompressionMethod( [ { compressionMethod => $method } ] )
 
 Get or set the member's C<desiredCompressionMethod>. This is
 the compression method that will be used when the member is
@@ -1653,9 +1698,9 @@ member desiredCompressionLevel to 0; changing to
 COMPRESSION_DEFLATED will change the member
 desiredCompressionLevel to COMPRESSION_LEVEL_DEFAULT.
 
-=item desiredCompressionLevel( [$method] )
-
+=item desiredCompressionLevel( [ $level ] )
 
+=item desiredCompressionLevel( [ { compressionLevel => $level } ] )
 
 Get or set the member's desiredCompressionLevel This is the
 method that will be used to write. Returns prior
@@ -1669,14 +1714,10 @@ to COMPRESSION_DEFLATED.
 
 =item externalFileName()
 
-
-
 Return the member's external file name, if any, or undef.
 
 =item fileName()
 
-
-
 Get or set the member's internal filename. Returns the
 (possibly new) filename. Names will have backslashes
 converted to forward slashes, and will have multiple
@@ -1684,15 +1725,11 @@ consecutive slashes converted to single ones.
 
 =item lastModFileDateTime()
 
-
-
 Return the member's last modification date/time stamp in
 MS-DOS format.
 
 =item lastModTime()
 
-
-
 Return the member's last modification date/time stamp,
 converted to unix localtime format.
 
@@ -1715,7 +1752,9 @@ header. This is only set for members read from a zip file.
 Return member attributes as read from the ZIP file. Note that
 these are NOT UNIX!
 
-=item unixFileAttributes( [$newAttributes] )
+=item unixFileAttributes( [ $newAttributes ] )
+
+=item unixFileAttributes( [ { attributes => $newAttributes } ] )
 
 Get or set the member's file attributes using UNIX file
 attributes. Returns old attributes.
@@ -1724,16 +1763,20 @@ attributes. Returns old attributes.
 
 Note that the return value has more than just the file
 permissions, so you will have to mask off the lowest bits for
-comparisions.
+comparisons.
 
-=item localExtraField( [$newField] )
+=item localExtraField( [ $newField ] )
+
+=item localExtraField( [ { field => $newField } ] )
 
 Gets or sets the extra field that was read from the local
 header. This is not set for a member from a zip file until
 after the member has been written out. The extra field must
 be in the proper format.
 
-=item cdExtraField( [$newField] )
+=item cdExtraField( [ $newField ] )
+
+=item cdExtraField( [ { field => $newField } ] )
 
 Gets or sets the extra field that was read from the central
 directory header. The extra field must be in the proper
@@ -1743,7 +1786,9 @@ format.
 
 Return both local and CD extra fields, concatenated.
 
-=item fileComment( [$newComment] )
+=item fileComment( [ $newComment ] )
+
+=item fileComment( [ { comment => $newComment } ] )
 
 Get or set the member's file comment.
 
@@ -1778,13 +1823,29 @@ external files until after the member has been written.
 
 Return the uncompressed size for this member.
 
+=item password( [ $password ] )
+
+Returns the password for this member to be used on decryption.
+If $password is given, it will set the password for the decryption.
+
 =item isEncrypted()
 
 Return true if this member is encrypted. The Archive::Zip
-module does not currently create or extract encrypted
-members.
+module does not currently support creation of encrypted
+members. Decryption works more or less like this:
 
-=item isTextFile( [$flag] )
+  my $zip = Archive::Zip->new;
+  $zip->read ("encrypted.zip");
+  for my $m (map { $zip->memberNamed ($_) } $zip->memberNames) {
+      $m->password ("secret");
+      $m->contents;  # is "" when password was wrong
+
+That shows that the password has to be set per member, and not per
+archive. This might change in the future.
+
+=item isTextFile( [ $flag ] )
+
+=item isTextFile( [ { flag => $flag } ] )
 
 Returns true if I am a text file. Also can set the status if
 given an argument (then returns old state). Note that this
@@ -1802,6 +1863,8 @@ format whether or not they came from a text file.
 
 =item extractToFileNamed( $fileName )
 
+=item extractToFileNamed( { name => $fileName } )
+
 Extract me to a file with the given name. The file will be
 created with default modes. Directories will be created as
 needed.
@@ -1848,7 +1911,9 @@ in chunks using these methods:
 
 =over 4
 
-=item readChunk( [$chunkSize] )
+=item readChunk( [ $chunkSize ] )
+
+=item readChunk( [ { chunkSize => $chunkSize } ] )
 
 This reads the next chunk of given size from the member's
 data stream and compresses or uncompresses it as necessary,
@@ -1865,7 +1930,7 @@ the last chunk, where it returns AZ_STREAM_END. Returns C<(
 
 Rewind data and set up for reading data streams or writing
 zip files. Can take options for C<inflateInit()> or
-C<deflateInit()>, but this isn't likely to be necessary.
+C<deflateInit()>, but this is not likely to be necessary.
 Subclass overrides should call this method. Returns C<AZ_OK>
 on success.
 
@@ -1876,7 +1941,7 @@ Must be called to close files, etc. Returns AZ_OK on success.
 
 =item readIsDone()
 
-Return true if the read has run out of data or errored out.
+Return true if the read has run out of data or encountered an error.
 
 =item contents()
 
@@ -1896,6 +1961,8 @@ change the class of the member):
 
 =item extractToFileHandle( $fh )
 
+=item extractToFileHandle( { fileHandle => $fh } )
+
 Extract (and uncompress, if necessary) the member's contents
 to the given file handle. Return AZ_OK on success.
 
@@ -1995,13 +2062,18 @@ Support for L<IO::Scalar> most likely will B<not> be restored in the
 future, as L<IO::Scalar> itself cannot change the way it is implemented
 due to back-compatibility issues.
 
+=head2 Wrong password for encrypted members
+
+When an encrypted member is read using the wrong password, you currently
+have to re-read the entire archive to try again with the correct password.
+
 =head1 TO DO
 
 * auto-choosing storing vs compression
 
 * extra field hooks (see notes.txt)
 
-* check for dups on addition/renaming?
+* check for duplicates on addition/renaming?
 
 * Text file extraction (line end translation)
 
@@ -2014,7 +2086,11 @@ due to back-compatibility issues.
 
 * Handle tainted paths correctly
 
-* Work on better compatability with other IO:: modules
+* Work on better compatibility with other IO:: modules
+
+* Support encryption
+
+* More user-friendly decryption
 
 =head1 SUPPORT
 
@@ -2036,7 +2112,7 @@ Originally by Ned Konz E<lt>nedkonz@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Some parts copyright 2006 - 2009 Adam Kennedy.
+Some parts copyright 2006 - 2012 Adam Kennedy.
 
 Some parts copyright 2005 Steve Peters.
 
@@ -2052,8 +2128,4 @@ read Zip archive members as if they were files.
 
 L<Compress::Raw::Zlib>, L<Archive::Tar>, L<Archive::Extract>
 
-There is a Japanese translation of this
-document at L<http://www.memb.jp/~deq/perl/doc-ja/Archive-Zip.html>
-that was done by DEQ E<lt>deq@oct.zaq.ne.jpE<gt> . Thanks! 
-
 =cut
