@@ -13,8 +13,14 @@ use Cwd;
 use vars qw( $VERSION @ISA );
 
 BEGIN {
-    $VERSION = '1.30';
+    $VERSION = '1.32';
     @ISA     = qw( Archive::Zip );
+
+    if ( $^O eq 'MSWin32' ) {
+        require Win32;
+        require Encode;
+        Encode->import( qw{ encode_utf8 decode_utf8 } );
+    }
 }
 
 use Archive::Zip qw(
@@ -24,6 +30,8 @@ use Archive::Zip qw(
   :UTILITY_METHODS
 );
 
+our $UNICODE;
+
 # Note that this returns undef on read errors, else new zip object.
 
 sub new {
@@ -117,7 +125,7 @@ sub zipfileComment {
     my $comment = $self->{'zipfileComment'};
     if (@_) {
         my $new_comment = ( ref( $_[0] ) eq 'HASH' ) ? shift->{comment} : shift;
-        $self->{'zipfileComment'} = pack( 'C0a*', $new_comment );    # avoid unicode
+        $self->{'zipfileComment'} = pack( 'C0a*', $new_comment );    # avoid Unicode
     }
     return $comment;
 }
@@ -245,6 +253,10 @@ sub addFile {
         ( $fileName, $newName, $compressionLevel ) = @_;
     }
 
+    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+        $fileName = Win32::GetANSIPathName($fileName);
+    }
+
     my $newMember = $self->ZIPMEMBERCLASS->newFromFile( $fileName, $newName );
     $newMember->desiredCompressionLevel($compressionLevel);
     if ( $self->{'storeSymbolicLink'} && -l $fileName ) {
@@ -255,6 +267,9 @@ sub addFile {
     } else {
         $self->addMember($newMember);
     }
+    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+        $newMember->{'fileName'} = encode_utf8( Win32::GetLongPathName($fileName) );
+    }
     return $newMember;
 }
 
@@ -290,6 +305,10 @@ sub addDirectory {
         ( $name, $newName ) = @_;
     }
 
+    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+        $name = Win32::GetANSIPathName($name);
+    }
+
     my $newMember = $self->ZIPMEMBERCLASS->newDirectoryNamed( $name, $newName );
     if ( $self->{'storeSymbolicLink'} && -l $name ) {
         my $link = readlink $name;
@@ -301,6 +320,9 @@ sub addDirectory {
     } else {
         $self->addMember($newMember);
     }
+    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+        $newMember->{'fileName'} = encode_utf8( Win32::GetLongPathName($name) );
+    }
     return $newMember;
 }
 
@@ -319,6 +341,10 @@ sub addFileOrDirectory {
         ( $name, $newName, $compressionLevel ) = @_;
     }
 
+    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+        $name = Win32::GetANSIPathName($name);
+    }
+
     $name =~ s{/$}{};
     if ( $newName ) {
         $newName =~ s{/$}{};
@@ -737,10 +763,19 @@ sub addTree {
         local $main::_ = $File::Find::name;
         my $dir = _untaintDir($File::Find::dir);
         chdir($startDir);
-        push( @files, $File::Find::name ) if (&$pred);
+        if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+            push( @files, Win32::GetANSIPathName($File::Find::name) ) if (&$pred);
+            $dir = Win32::GetANSIPathName($dir);
+        }
+        else {
+            push( @files, $File::Find::name ) if (&$pred);
+        }
         chdir($dir);
     };
 
+    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+        $root = Win32::GetANSIPathName($root);
+    }
     File::Find::find( $wanted, $root );
 
     my $rootZipName = _asZipDirName( $root, 1 );    # with trailing slash
@@ -749,7 +784,13 @@ sub addTree {
     $dest = _asZipDirName( $dest, 1 );              # with trailing slash
 
     foreach my $fileName (@files) {
-        my $isDir = -d $fileName;
+        my $isDir;
+        if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
+            $isDir = -d Win32::GetANSIPathName($fileName);
+        }
+        else {
+            $isDir = -d $fileName;
+        }
 
         # normalize, remove leading ./
         my $archiveName = _asZipDirName( $fileName, $isDir );
@@ -810,7 +851,15 @@ sub extractTree {
     }
 
     $root = '' unless defined($root);
-    $dest = './' unless defined($dest);
+    if ( defined $dest ) {
+        if ( $dest !~ m{/$} ) {
+            $dest .= '/';
+        }
+    }
+    else {
+        $dest = './';
+    }
+
     my $pattern = "^\Q$root";
     my @members = $self->membersMatching($pattern);
 
@@ -898,7 +947,7 @@ sub updateMember {
 # whether the file or directory already exists in the zip file.
 #
 # If the fourth argument $mirror is true, then delete all my members
-# if corresponding files weren't found.
+# if corresponding files were not found.
 
 sub updateTree {
     my $self = shift;
