@@ -40,7 +40,7 @@ use vars qw{$VERSION};
 my $nl;
 
 BEGIN {
-    $VERSION = '1.30';
+    $VERSION = '1.32';
     $VERSION = eval $VERSION;
      # Requirement for newline conversion. Should check for e.g., DOS and OS/2 as well, but am too lazy.
      $nl = $^O eq 'MSWin32' ? "\r\n" : "\n";
@@ -119,7 +119,7 @@ sub set_compression {
 Set the line end character to use. This is set to \n by default
 except on Windows systems where it is set to \r\n. You will
 only need to set this on systems which are not Windows or Unix
-based and require a line end diffrent from \n.
+based and require a line end different from \n.
 This is a class method so call as C<Archive::Zip::MemberRead>->C<setLineEnd($nl)>
 
 =cut
@@ -154,7 +154,7 @@ sub _reset_vars {
 
 =item input_record_separator(expr)
 
-If the argumnet is given, input_record_separator for this
+If the argument is given, input_record_separator for this
 instance is set to it. The current setting (which may be
 the global $/) is always returned.
 
@@ -247,20 +247,37 @@ Returns undef on eof. All subsequent calls would return undef,
 unless a rewind() is called.
 Note: The line returned has the input_record_separator (default: newline) removed.
 
+=item getline( { preserve_line_ending => 1 } )
+
+Returns the next line including the line ending.
+
 =cut
 
 sub getline {
-    my $self = shift;
+    my ( $self, $argref ) = @_;
+
     my $size = $self->buffer_size();
     my $sep  = $self->_sep_re();
 
+    my $preserve_line_ending;
+    if ( ref $argref eq 'HASH' ) {
+        $preserve_line_ending = $argref->{'preserve_line_ending'};
+        $sep =~ s/\\([^A-Za-z_0-9])+/$1/g;
+    }
+
     for (;;) {
         if (   $sep
             && defined($self->{buffer})
             && $self->{buffer} =~ s/^(.*?)$sep//s
            ) {
+            my $line = $1;
             $self->{line_no}++;
-            return $1;
+            if ($preserve_line_ending) {
+                return $line . $sep;
+            }
+            else {
+                return $line;
+            }
         } elsif ($self->{at_end}) {
             $self->{line_no}++ if $self->{buffer};
             return delete $self->{buffer};
@@ -317,7 +334,8 @@ sub read {
 
 =head1 AUTHOR
 
-Sreeji K. Das, <sreeji_k@yahoo.com>
+Sreeji K. Das E<lt>sreeji_k@yahoo.comE<gt>
+
 See L<Archive::Zip> by Ned Konz without which this module does not make
 any sense! 
 
